@@ -105,3 +105,80 @@ mdl_time_refit.mdl_time_ensemble_model_spec <- function(object, data, ..., contr
     }
 
 }
+
+# 3.0 RECURSIVE ----
+
+#' @export
+mdl_time_refit.recursive_ensemble <- function(object, data, ..., control = NULL) {
+
+    if (inherits(object, "recursive")) {
+
+        # Get transformer
+        transformer <- object$spec$transform
+
+        # Create new train tail
+        train_tail_old <- object$spec$train_tail
+
+        train_tail_new <- data %>%
+            dplyr::slice_tail(n = nrow(train_tail_old))
+
+        # Refit as normal ensemble
+        # object$spec   <- NULL
+        class(object) <- class(object)[3:length(class(object))]
+        object        <- mdl_time_refit(object, data, ..., control = control)
+
+        # Make Recursive
+        object <- recursive(object, transform = transformer, train_tail = train_tail_new)
+
+        # Need to overwrite transformer
+        object$spec$transform <- transformer
+
+
+    } else {
+
+        # Get transformer
+        transformer <- object$spec$transform
+
+        # Create new train tail
+        train_tail_old <- object$spec$train_tail
+
+        # print("Spec ID")
+        # print(object$spec$id)
+
+        n <- object$spec$train_tail %>%
+            dplyr::count(!! rlang::sym(object$spec$id)) %>%
+            dplyr::pull(n) %>%
+            stats::median(na.rm = TRUE)
+
+        train_tail_new <- data %>%
+            panel_tail(
+                id = !! object$spec$id,
+                n  = n
+            )
+
+        id_old <- object$spec$id
+
+        # Refit as normal ensemble
+        object$spec   <- NULL
+        class(object) <- class(object)[3:length(class(object))]
+        object        <- mdl_time_refit(object, data, ..., control = control)
+
+        # print("ID")
+        # print(id_old)
+
+        # Make Recursive
+        object <- recursive(
+            object,
+            transform  = transformer,
+            train_tail = train_tail_new,
+            id         = id_old
+        )
+
+        # Need to overwrite transformer
+        object$spec$transform <- transformer
+
+    }
+
+    return(object)
+
+}
